use r2d2::ManageConnection;
use std::{io::Error, net::{SocketAddr, TcpStream}};
use std::io::Write;

pub struct PixelflutConnectionManager {
    addr: SocketAddr
}

pub struct PixelflutConnection {
    stream: TcpStream
}

impl PixelflutConnectionManager {
    pub fn new(addr: &str) -> Result<Self, Error> {
        let addr: SocketAddr = match addr.parse() {
            Ok(addr) => addr,
            Err(e) => panic!("Error in parsing the socket address: {}", e)
        };
        Ok(Self {addr})
    }

    fn get_connection(&self) -> Result<PixelflutConnection, Error> {
        let stream = TcpStream::connect(self.addr)?;
        Ok(PixelflutConnection {stream})
    }

}

pub struct Rgba(pub u32);

impl From<u32> for Rgba {
    fn from(a: u32) -> Self {
        Self(a)
    }
}

impl PixelflutConnection {
    #[inline]
    pub fn new(addr: SocketAddr) -> Result<Self, Error> {
        let stream = TcpStream::connect(addr)?;
        Ok(Self {stream})
    }

    pub fn set(&mut self, x: usize, y: usize, c: impl Into<Rgba>) {
        let msg = format!("PX {} {} {}\n", x,y,12);
        self.stream.write(&msg[..].as_bytes());
    }
}


impl ManageConnection for PixelflutConnectionManager {
    type Connection = PixelflutConnection;
    type Error = Error;

    fn connect(&self) -> Result<Self::Connection, Self::Error> {
        self.get_connection()
    }

    fn is_valid(&self, conn: &mut Self::Connection) -> Result<(), Self::Error> {
        todo!()
    }

    fn has_broken(&self, conn: &mut Self::Connection) -> bool {
        false
    }

}
